#!/usr/bin/env python3

"""
Mariadb controller

The script is responsible for set mariadb_role: primary to first
active pod in mariadb deployment.

Env variables:
MARIADB_CONTROLLER_DEBUG: Flag to enable debug when set to 1.
MARIADB_CONTROLLER_CHECK_PODS_DELAY: The delay between check pod attempts.
MARIADB_CONTROLLER_PYKUBE_REQUEST_TIMEOUT: The timeout for kubernetes http session
MARIADB_CONTROLLER_PODS_NAMESPACE: The namespace to look for mariadb pods.

Changelog:
0.1.0: Initial varsion
"""


import logging
import os
import sys
import time

import pykube

MARIADB_CONTROLLER_DEBUG = os.getenv("MARIADB_CONTROLLER_DEBUG")
MARIADB_CONTROLLER_CHECK_PODS_DELAY = int(
    os.getenv("MARIADB_CONTROLLER_CHECK_PODS_DELAY", 10)
)
MARIADB_CONTROLLER_PYKUBE_REQUEST_TIMEOUT = int(
    os.getenv("MARIADB_CONTROLLER_PYKUBE_REQUEST_TIMEOUT", 60)
)
MARIADB_CONTROLLER_PODS_NAMESPACE = os.getenv(
    "MARIADB_CONTROLLER_PODS_NAMESPACE", "openstack"
)

log_level = "DEBUG" if MARIADB_CONTROLLER_DEBUG else "INFO"
logging.basicConfig(
    stream=sys.stdout,
    format="%(asctime)s %(levelname)s %(name)s %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)
LOG = logging.getLogger("mariadb-controller")

LOG.setLevel(log_level)


def login():
    config = pykube.KubeConfig.from_env()
    client = pykube.HTTPClient(
        config=config, timeout=MARIADB_CONTROLLER_PYKUBE_REQUEST_TIMEOUT
    )
    LOG.info(f"Created k8s api client from context {config.current_context}")
    return client


api = login()


def resource_list(klass, selector, namespace=None):
    return klass.objects(api).filter(namespace=namespace, selector=selector)


def get_mariadb_pods():
    sorted_pods = sorted(
        resource_list(
            pykube.Pod,
            {"application": "mariadb", "component": "server"},
            MARIADB_CONTROLLER_PODS_NAMESPACE,
        ).iterator(),
        key=lambda i: i.name,
    )
    return sorted_pods


def ensure_mariadb_role(pod, role):
    if pod.obj["metadata"]["labels"].get("mariadb_role") == role:
        LOG.debug(f"Nothing to do, pod's {pod.name} role is {role}")
    else:
        pod.obj["metadata"]["labels"]["mariadb_role"] = role
        pod.update()
        LOG.info(f"Set pod {pod.name} role to {role}")


def is_ready(pod):
    if pod.ready and "deletionTimestamp" not in pod.metadata:
        return True


def main():
    while True:
        has_ready = False

        for pod in get_mariadb_pods():
            pod.reload()
            if has_ready is False and is_ready(pod):
                ensure_mariadb_role(pod, "primary")
                has_ready = True
            else:
                ensure_mariadb_role(pod, "backup")
        LOG.debug(f"Sleeping for {MARIADB_CONTROLLER_CHECK_PODS_DELAY}")
        time.sleep(MARIADB_CONTROLLER_CHECK_PODS_DELAY)


main()
